﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.ComponentModel.DataAnnotations;

namespace Library.DataModel {

    public static class BookRules {

        public static ValidationResult EnsureMeaningfulDescription(string description) {
            if (String.IsNullOrEmpty(description)) {
                return new ValidationResult("Every book needs a description.");
            }

            string[] words = description.Split(new char[] { ' ', '.' },
                                               StringSplitOptions.RemoveEmptyEntries);
            if (words.Length < 10) {
                return new ValidationResult("Every book needs a good description. Please supply a paragraph of text.");
            }

            return ValidationResult.Success;
        }

        public static ValidationResult EnsureOnShelfIsNotMarkedMissing(Book book) {
            if (book.OnShelf && book.Missing) {
                return new ValidationResult("A book on shelf cannot be marked as missing as well.");
            }

            return ValidationResult.Success;
        }
    }
}
