﻿using System;
using System.Collections.Generic;
using System.Data.Services;
using System.Linq;
using System.ServiceModel.Web;
using System.Web;
using System.Data.Services.Providers.DomainService;
using System.Security.Principal;

namespace Library.Web {

    public abstract class DataService<TDomainService> : System.Data.Services.DataService<TDomainService>, IServiceProvider {

        // Plumbing code... will go away soon
        private IServiceProvider provider;

        public object GetService(Type serviceType) {
            if (this.provider == null) {
                Type dataServiceType = this.GetType();
                while (dataServiceType != null && dataServiceType != typeof(object)) {
                    if (!dataServiceType.IsGenericType || dataServiceType.GetGenericTypeDefinition() != typeof(DataService<>)) {
                        dataServiceType = dataServiceType.BaseType;
                        continue;
                    }

                    Type domainServiceType = dataServiceType.GetGenericArguments()[0];
                    Type domainServiceDataProviderType = typeof(DomainServiceDataProvider<>).MakeGenericType(domainServiceType);
                    this.provider = (IServiceProvider)Activator.CreateInstance(domainServiceDataProviderType, this);
                    break;
                }
            }

            object target = this.provider.GetService(serviceType);
            if (target == null) {
                if (serviceType == typeof(IPrincipal)) {
                    return HttpContext.Current.User;
                }
            }

            return target;
        }
    }
}
