﻿// DomainContextSyncExtensions.cs
//

using System;
using System.Windows;
using System.Windows.Ria.Data;

namespace System.Windows.Ria.Sync {

    public static class DomainContextSyncExtensions {

        public static void AttachSyncBehavior<TDomainContext>(this TDomainContext domainContext, DomainSyncBehavior<TDomainContext> syncBehavior) where TDomainContext : DomainContext {
            if (domainContext == null) {
                throw new ArgumentNullException("domainContext");
            }
            if (syncBehavior == null) {
                throw new ArgumentNullException("syncBehavior");
            }
            if (syncBehavior.DomainContext != null) {
                throw new ArgumentException("The sync behavior is already attached to a different domain context.");
            }

            syncBehavior.Attach(domainContext);
            syncBehavior.Sync(/* performLoad */ true, /* performSave */ false);
        }

        public static void DetachSyncBehavior<TDomainContext>(this TDomainContext domainContext, DomainSyncBehavior<TDomainContext> syncBehavior) where TDomainContext : DomainContext {
            if (domainContext == null) {
                throw new ArgumentNullException("domainContext");
            }
            if (syncBehavior == null) {
                throw new ArgumentNullException("syncBehavior");
            }

            if (syncBehavior.DomainContext != domainContext) {
                throw new ArgumentException("The sync behavior is associated with a different domain context.");
            }

            syncBehavior.Detach();
        }
    }
}
