﻿// DomainSyncBehavior.cs
//

using System;
using System.Windows;
using System.Windows.Ria.Data;
using System.Windows.Threading;

namespace System.Windows.Ria.Sync {

    public abstract class DomainSyncBehavior<TDomainContext> where TDomainContext : DomainContext {

        private TDomainContext _domainContext;
        private DispatcherTimer _timer;

        protected DomainSyncBehavior(TimeSpan syncInterval) {
            _timer = new DispatcherTimer();
            _timer.Interval = syncInterval;
            _timer.Tick += OnTimerTick;
        }

        protected internal TDomainContext DomainContext {
            get {
                return _domainContext;
            }
        }

        internal void Attach(TDomainContext domainContext) {
            _domainContext = domainContext;
            OnAttach();
        }

        internal void Detach() {
            OnDetach();
            _domainContext = null;
        }

        public void Load() {
            Sync(/* performLoad */ true, /* performSave */ false);
        }

        protected abstract void OnAttach();

        protected abstract void OnDetach();

        private void OnTimerTick(object sender, EventArgs e) {
            _timer.Stop();
            Sync(/* performLoad */ true, /* performSave */ true);
        }

        public void Save() {
            Sync(/* performLoad */ false, /* performSave */ true);
        }

        public virtual void StartSync() {
            _timer.Start();
        }

        public virtual void StopSync() {
            _timer.Stop();
        }

        protected internal abstract void Sync(bool performLoad, bool performSave);
    }
}
